/*
Copyright 2006 Frank van der Loo

This file is part of Paratim.

Paratim is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Paratim is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Paratim; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "SDL.h"
#include "chopper.h"
#include "llist.h"

// Create a new chopper that flies to the left
chopper::chopper(int x, Direction d) : sprite(DATADIR, d == LEFT ? "/heli1.bmp" : "/heli2.bmp")
{
	dst.x = x;
	if (d == LEFT)
	{
		dst.y = LEFT_CHOPPER_Y;
	}
	else
	{
		dst.y = RIGHT_CHOPPER_Y;
	}
	dst.w = image->w;
	dst.h = image->h;
	black.w = image->w + 2*CHOPPER_SPEED;
	black.h = image->h + CHOPPER_SPEED;
	black.x = dst.x - CHOPPER_SPEED;
	black.y = dst.y - CHOPPER_SPEED;
	points = 10;
	end = false;
	dir = d;
}

// Move the chopper "plane" a few pixels to the left
void chopper::move(bool plane, SDL_Surface *screen)
{
	SDL_Rect dsttemp = dst;
	SDL_Rect blacktmp = black;
	SDL_FillRect(screen, &blacktmp, SDL_MapRGB(screen->format, 0, 0, 0));
	SDL_BlitSurface(this->getSurface(), NULL, screen, &dsttemp);
	if (dir == LEFT)
	{
		if (dst.x < 0-image->w)
		{
			if (plane)
				return;
			dst.x = WIDTH+image->w;
		}
	}
	else
	{
		if (dst.x > WIDTH)
		{
			if (plane)
				return;
			dst.x = 0-image->w;
		}
	}
	if (dir == LEFT)
	{
		dst.x-=CHOPPER_SPEED;
	}
	else
	{
		dst.x+=CHOPPER_SPEED;
	}
	black.x = dst.x - CHOPPER_SPEED;
}

// Handles the events when a chopper has been hit
void chopper::hit(SDL_Surface *screen)
{
	end = true;
	SDL_Rect blacktmp = black;
	SDL_FillRect(screen, &blacktmp, SDL_MapRGB(screen->format, 0, 0, 0));
}

// Decide wether to drop a trooper and drop one
bool chopper::drop_trooper()
{
	if ((dst.x < (GUN_BEGIN-38) || dst.x > GUN_END) && dst.x > 0 && dst.x < (WIDTH-38))
	{
		int random = (rand()%TROOPER_VALUE)+1;
		//std::cout << std::endl << random << std::endl;
		return (random == 1);
		//return (((rand()%100)+1) == 1);
	}
	else
		return false;
}

// Checks if the chopper is visible on the screen
bool chopper::visible()
{
	return dst.x < WIDTH && dst.x+dst.w > 0;
}

// moves the choppers
void move_choppers(SDL_Surface *screen, chopper *chopperar[NUM_OF_CHOP], linkedList2 &troopLList, bool plane)
{
	for (int i=0;i<NUM_OF_CHOP;i++)
	{
		if (chopperar[i])
		{
			if (chopperar[i]->drop_trooper())
			{
				#ifdef DEBUG
				  std::cout << "Trooper dropped left: " << chopperar[i]->getDstRect().x << " : " << troopLList.getSize() << "\n\n\n";
				#endif
				trooper *troop = new trooper(chopperar[i]->getDstRect().x, chopperar[i]->getDstRect().y);
				troopLList.add(troop);
			}
			chopperar[i]->move(plane, screen);
			if (chopperar[i]->getEnd())
			{
				free(chopperar[i]);
				chopperar[i] = NULL;
			}
		}
		else
		{
			bool overlay = false;
			if (i < NUM_OF_CHOP/2)
			{
				for (int j=0;j<NUM_OF_CHOP;j++)
				{
					if (chopperar[j] && ((((chopperar[j]->getDstRect().x)+((chopperar[j]->getSurface())->w)) >= 950) || (((chopperar[j]->getDstRect().x) <= 137))))
						overlay = true;
				}
				if (!overlay && !plane)
				{
					chopperar[i] = new chopper(950, LEFT);
				}
			}
			else
			{
				overlay = false;
				for (int j=0;j<NUM_OF_CHOP;j++)
				{
					if (chopperar[j] && (((chopperar[j]->getDstRect().x) >= -150 && (chopperar[j]->getDstRect().x) <= 20) || (chopperar[j]->getDstRect().x) >= 650))
						overlay = true;
				}
				if (!overlay && !plane)
				{
					chopperar[i] = new chopper(-150, RIGHT);
				}
			}
		}
	}
}
