/*
Copyright 2006 Frank van der Loo

This file is part of Paratim.

Paratim is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Paratim is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Paratim; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <ctime>
#include <cstdlib>
#include "SDL.h" 

#include "sprite.h"
#include "gun.h"
#include "bullet.h"
#include "chopper.h"
#include "trooper.h"
#include "llist.h"
#include "bomb.h"
#include "plane.h"
#include "txt2sdl.h"
#include "menu.h"

using namespace std;

void move_choppers(SDL_Surface *screen, chopper *chopperar[NUM_OF_CHOP], linkedList2 &troopLList, bool plane);
int move_bullet(SDL_Surface *screen, linkedList &LList, linkedList2 &troopLList, chopper *chopperar[NUM_OF_CHOP], plane **airplane, bomb **plane_bomb);
void move_plane(SDL_Surface *screen, plane *airplane, bomb *plane_bomb);

// Checks if any choppers are visible
bool visible(chopper *chopperar[])
{
	bool vis = false;
	for (int i=0;i<NUM_OF_CHOP;i++)
	{
		if (chopperar[i])
		{
			if (chopperar[i]->visible())
			{
				vis = true;
			}
		}
	}
	return vis;
}

// Checks if any paratroopers are visible
bool visible(linkedList2 &troopLList)
{
	return troopLList.getSize();
}

// Shows the current score and hi-score
void show_scores(SDL_Surface *screen, SDL_Surface *alph, int points, int high_score)
{
	SDL_Surface *line;
	int score_end, high_score_end;

	char *temppath = new char[strlen(DATADIR)+strlen("/line.bmp")+1];
	strcpy(temppath, DATADIR);
	temppath = strcat(temppath, "/line.bmp");
	line = SDL_LoadBMP(temppath);
	if (line == NULL)
	{
		std::cout << "Couldn't load " << temppath << std::endl;
		exit(EXIT_FAILURE);
	}
	delete temppath;
	temppath = NULL;
	SDL_Rect bottomline;
	bottomline.x = 0;
	bottomline.y = HEIGHT-50;
	SDL_BlitSurface(line, NULL, screen, &bottomline);

	score_end = txt2sdl("SCORE: ", 5, HEIGHT-45, alph, screen);
	high_score_end = txt2sdl("HI SCORE: ", WIDTH/2+5, HEIGHT-45, alph, screen);
	SDL_Rect blackscore, blackhiscore;
	blackscore.x = score_end;
	blackscore.y = HEIGHT-45;
	blackscore.h = 45;
	blackscore.w = WIDTH/2 - score_end;
	blackhiscore.x = high_score_end;
	blackhiscore.y = HEIGHT-45;
	blackhiscore.h = 45;
	blackhiscore.w = WIDTH - high_score_end;
	SDL_FillRect(screen, &blackscore, SDL_MapRGB(screen->format, 0, 0, 0));
	SDL_FillRect(screen, &blackhiscore, SDL_MapRGB(screen->format, 0, 0, 0));
	int2sdl(points, score_end, HEIGHT-45, alph, screen);
	int2sdl(high_score, high_score_end, HEIGHT-45, alph, screen);
}

// The function that runs the game
void main_game(SDL_Surface *screen, SDL_Surface *alph, int &high_score)
{
	int points = 0;
	bool left_gun=false;
	bool right_gun=false;
	bool delay = false;
	Uint8 *keys;
	Uint32 current_time, oldtime = SDL_GetTicks(), old_plane_start_time = 0;
	unsigned int random_time = rand()%5+1;

	SDL_Event event;
	unsigned int mins_to_planes = /*rand()%RND_MINS_TO_PLANES +*/ 1;
	//cout << "\nmins_to_planes: " << mins_to_planes << endl;

	linkedList LList;
	linkedList2 troopLList;

	bullet *newbul = NULL;

	gun player_gun(DATADIR, "/parat-gun.bmp");
	chopper *chopperar[NUM_OF_CHOP];
	for (int i=0;i<NUM_OF_CHOP;i++)
	{
		if (i < NUM_OF_CHOP/2)
		{
			chopperar[i] = new chopper(WIDTH+(i*(WIDTH/NUM_OF_CHOP)), LEFT);
		}
		else
		{
			chopperar[i] = new chopper(0-chopperar[0]->getW()-(i*(WIDTH/NUM_OF_CHOP)), RIGHT);
		}
	}

	trooper* landed_troopers[20];
	for (int i=0;i<20;i++)
		landed_troopers[i] = NULL;

	plane *planes[5+1];
	for (int i=0;i<5+1;i++)
	{
		planes[i] = NULL;
	}
	int num_of_planes = get_num_of_planes();
	set_planes(planes, num_of_planes);

	bomb *plane_bomb = new bomb();
	int plane_num = 0;

	bool end = false, planes_approaching = false;
	SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 0, 0, 0));
	// <main loop>
	while(!end)
	{
		current_time = SDL_GetTicks();
		if (current_time - old_plane_start_time >= 30000)//mins_to_planes*60*1000 && !plane)
		{
			planes_approaching = true;
			mins_to_planes = rand()%RND_MINS_TO_PLANES + 1;
			//cout << "\nmins_to_planes: " << mins_to_planes << endl;
		}
		//cout << "\ncurrent_time - old_plane_start_time = " << current_time - old_plane_start_time << endl;
		keys = SDL_GetKeyState(NULL);
		
		show_scores(screen, alph, points, high_score);
		showGun(screen, player_gun);
		
	// <chopper>
		move_choppers(screen, chopperar, troopLList, planes_approaching);
	// </chopper>
	// <plane>
		if (plane_bomb == NULL)
			plane_bomb = new bomb();
		if (delay)
		{
			if (SDL_GetTicks()-oldtime >= (random_time*1000))
				delay = false;
			move_bomb(screen, plane_bomb);
		}
		else if (planes_approaching && !visible(chopperar) && !visible(troopLList))		//make sure all choppers and troopers are gone before the planes start
		{

			plane *left = planes[plane_num];
			move_plane(screen, left, plane_bomb);

			if (((left && left->getEnd()) || (!left)) && !delay)
			{
				delete planes[plane_num];
				planes[plane_num] = NULL;
				plane_num++;
				oldtime = SDL_GetTicks();
				random_time = rand()%5+1;
				delay = true;
			}
			if (plane_num >= num_of_planes)
			{
				planes_approaching = false;
				plane_num = 0;
				num_of_planes = get_num_of_planes();
				set_planes(planes, num_of_planes);
				old_plane_start_time = current_time;
			}
	// </plane>
	// <bomb>
			if (move_bomb(screen, plane_bomb))
				end = true;
		}
	// </bomb>
	// <trooper>
		points += move_troopers(troopLList, screen, landed_troopers);
	// </trooper>
	// <bullet>
		points += move_bullet(screen, LList, troopLList, chopperar, &planes[plane_num], &plane_bomb);
	// </bullet>

		SDL_Flip(screen);
		if (four_troopers(landed_troopers))
			end = true;
		if (SDL_PollEvent(&event) || left_gun || right_gun)
		{
			SDL_Rect gunblacktemp = player_gun.getBlack();
			if (keys[SDLK_LEFT] || left_gun)
			{
				left_gun = true;
				right_gun = false;
				SDL_FillRect(screen, &gunblacktemp, SDL_MapRGB(screen->format, 0, 0, 0));
				player_gun.left();
			}
			if (keys[SDLK_RIGHT] || right_gun)
			{
				left_gun = false;
				right_gun = true;
				SDL_FillRect(screen, &gunblacktemp, SDL_MapRGB(screen->format, 0, 0, 0));
				player_gun.right();
			}
			if (keys[SDLK_UP])
			{
				left_gun = false;
				right_gun = false;
				double x_direction = player_gun.getAngle()/900.0;
				double y_direction = 1.0-dabs(x_direction);
				newbul = new bullet();
		
				newbul->set_x_dir(x_direction);
				newbul->set_y_dir(y_direction);
		
		
				LList.add(newbul);
				if (points > 0)
					points--;
			}
			if (event.type == SDL_QUIT)
			{
				cout << "\nPoints: " << points << endl;
				exit(EXIT_SUCCESS);
			}
		}
	}
	SDL_Event endEvent;
	SDL_Rect gameOverBox;
	gameOverBox.w = 450;
	gameOverBox.h = 45;
	gameOverBox.x = 275;
	gameOverBox.y = 277;
	SDL_FillRect(screen, &gameOverBox, SDL_MapRGB(screen->format, 0, 0, 0));
	txt2sdl("GAME OVER", gameOverBox.x, gameOverBox.y, alph, screen);
	if (points > high_score)
	{
		high_score = points;
	}
	SDL_Flip(screen);
	while (endEvent.type != SDL_KEYDOWN)
		SDL_WaitEvent(&endEvent);
	// </main loop>
}

// Main function changed to this for Mac OS X
// Thanks to Nil Dogger for telling me this
extern "C" int main(int argc, char *argv[])
{
	if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE) == -1)
	{
		cout << "Error while initializing SDL!\n";
		exit(EXIT_FAILURE);
	}
	SDL_Surface *screen = SDL_SetVideoMode(WIDTH, HEIGHT, BPP, SDL_DOUBLEBUF);
	if (screen == NULL)
	{
		cout << "Error while opening a videomode of " << WIDTH << "x" << HEIGHT << "@" << BPP << endl;
		exit(EXIT_FAILURE);
	}

	SDL_WM_SetCaption("Paratim", "Paratim");
	srand(time(NULL));
	atexit(SDL_Quit);
	SDL_Surface *alph;
	char *temppath = new char[strlen(DATADIR)+strlen("/alfabet.bmp")+1];
	strcpy(temppath, DATADIR);
	temppath = strcat(temppath, "/alfabet.bmp");
	alph = SDL_LoadBMP(temppath);
	if (alph == NULL)
	{
		std::cout << "Couldn't load " << temppath << std::endl;
		exit(EXIT_FAILURE);
	}
	delete temppath;
	temppath = NULL;
	SDL_SetColorKey(alph, SDL_SRCCOLORKEY, SDL_MapRGB(alph->format, 255, 255, 0));
	menu mainMenu = create_main_menu();
	bool end = false;
	int high_score = 0;
	while (!end)
	{
		SDL_Event event;
		mainMenu.show(screen, alph, 300, 300);
		SDL_Flip(screen);
		Uint8 *keys = SDL_GetKeyState(NULL);
		if (SDL_PollEvent(&event))
		{
			if (keys[SDLK_UP])
			{
				mainMenu.up();
			}
			if (keys[SDLK_DOWN])
			{
				mainMenu.down();
			}
			if (keys[SDLK_RETURN])
			{
				string choice = mainMenu.getChoice();
				if (choice == "NEW GAME")
				{
					main_game(screen, alph, high_score);
				}
				else if (choice == "QUIT")
				{
					return 0;
				}
			}
			if (event.type == SDL_QUIT)
			{
      				return 0;
			}
		}
	}
 return 0;
}
