/*
Copyright 2006 Frank van der Loo

This file is part of Paratim.

Paratim is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Paratim is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Paratim; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef SPRITE_H
#define SPRITE_H

#include <iostream>
#include "SDL.h"

const int WIDTH = 800;
const int HEIGHT = 600;
const int BPP = 16;
const int GUN_BEGIN = 350;
const int GUN_END = 450;
const int TROOPER_VALUE = 500;
const int RND_MINS_TO_PLANES = 1;

enum Direction {LEFT, RIGHT};

class sprite
{
protected:
	SDL_Surface *image;
	SDL_Rect dst, black;
	int speed;

public:
	sprite(const char path[], const char file[])
	{
		char *temppath = new char[strlen(path)+strlen(file)+1];
		strcpy(temppath, path);
		temppath = strcat(temppath, file);
		//std::cout << "In sprite constructor\n";
		image = SDL_LoadBMP(temppath);
		if (image == NULL)
		{
			std::cout << "Couldn't load " << temppath << std::endl;
			exit(EXIT_FAILURE);
		}
		delete temppath;
		temppath = NULL;
		//std::cout << "After loadBMP\n";
		SDL_SetColorKey(image, SDL_SRCCOLORKEY, SDL_MapRGB(image->format, 255, 255, 0));
		//std::cout << "Leaving sprite constructor\n";
	}

	~sprite()
	{
		//std::cout << "In sprite destructor\n";
		//SDL_FreeSurface(image);
	}
	SDL_Rect getDstRect() {return dst;};
	void setDstRect(SDL_Rect rect){dst = rect;};
	SDL_Surface *getSurface() {return image;};
	SDL_Rect getBlack() {return black;};
	int getX() { return dst.x; }
	int getY() { return dst.y; }
	int getW() { return dst.w; }
	int getH() { return dst.h; }
	int getSpeed() { return speed; }
	void setSpeed(int s) { speed = s; }
};
#endif
